#include <X11/Xlibint.h>
#include <X11/Xproto.h>
#include <X11/extensions/extutil.h>
#include <X11/extensions/dri2proto.h>
#include <X11/extensions/Xfixes.h>

static char dri2ExtensionName[] = DRI2_NAME;

static XExtensionInfo *dri2Info = NULL;

static XEXT_GENERATE_CLOSE_DISPLAY(DRI2CloseDisplay, dri2Info);

static Bool
DRI2Error(Display *dpy, xError *err, XExtCodes *codes, int *ret_code)
{
  *ret_code = 0;

  if (err->majorCode == codes->major_opcode &&
      err->minorCode == X_DRI2DestroyDrawable &&
      err->errorCode == BadDrawable)
  {
    return True;
  }

  return False;
}

static XExtensionHooks dri2ExtensionHooks = {
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  NULL,
  DRI2CloseDisplay,
  NULL,
  NULL,
  DRI2Error,
  NULL
};

static XEXT_GENERATE_FIND_DISPLAY(DRI2FindDisplay,
                                  dri2Info,
                                  dri2ExtensionName,
                                  &dri2ExtensionHooks,
                                  0, NULL);

void
DRI2DestroyDrawable(Display *dpy, XID drawable)
{

  XExtDisplayInfo *info = DRI2FindDisplay(dpy);
  xDRI2DestroyDrawableReq *req;

  XextSimpleCheckExtension(dpy, info, dri2ExtensionName);

  XSync(dpy, False);

  LockDisplay(dpy);
  GetReq(DRI2DestroyDrawable, req);
  req->reqType = info->codes->major_opcode;
  req->dri2ReqType = X_DRI2DestroyDrawable;
  req->drawable = drawable;
  UnlockDisplay(dpy);

  SyncHandle();
}

Bool
DRI2QueryExtension(Display * dpy, int *eventBase, int *errorBase)
{
   XExtDisplayInfo *info = DRI2FindDisplay(dpy);

   if (XextHasExtension(info))
   {
      *eventBase = info->codes->first_event;
      *errorBase = info->codes->first_error;
      return True;
   }

   return False;
}

void
DRI2CopyRegion(Display * dpy, XID drawable, XserverRegion region,
               CARD32 dest, CARD32 src)
{
   XExtDisplayInfo *info = DRI2FindDisplay(dpy);
   xDRI2CopyRegionReq *req;
   xDRI2CopyRegionReply rep;

   XextSimpleCheckExtension(dpy, info, dri2ExtensionName);

   LockDisplay(dpy);
   GetReq(DRI2CopyRegion, req);
   req->reqType = info->codes->major_opcode;
   req->dri2ReqType = X_DRI2CopyRegion;
   req->drawable = drawable;
   req->region = region;
   req->dest = dest;
   req->src = src;

   _XReply(dpy, (xReply *) & rep, 0, xFalse);

   UnlockDisplay(dpy);
   SyncHandle();
}

void
DRI2CreateDrawable(Display * dpy, XID drawable)
{
   XExtDisplayInfo *info = DRI2FindDisplay(dpy);
   xDRI2CreateDrawableReq *req;

   XextSimpleCheckExtension(dpy, info, dri2ExtensionName);

   LockDisplay(dpy);
   GetReq(DRI2CreateDrawable, req);
   req->reqType = info->codes->major_opcode;
   req->dri2ReqType = X_DRI2CreateDrawable;
   req->drawable = drawable;
   UnlockDisplay(dpy);
   SyncHandle();
}
