/******************************************************************************
 * Name         : sgxfeaturedefs.h
 * Title        : SGX fexture definitions
 *
 * Copyright    : 2006-2007 by Imagination Technologies Limited.
 *              : All rights reserved. No part of this software, either
 *              : material or conceptual may be copied or distributed,
 *              : transmitted, transcribed, stored in a retrieval system or
 *              : translated into any human or computer language in any form
 *              : by any means, electronic, mechanical, manual or otherwise,
 *              : or disclosed to third parties without the express written
 *              : permission of Imagination Technologies Limited,
 *              : Home Park Estate, Kings Langley, Hertfordshire,
 *              : WD4 8LZ, U.K.
 *
 * Platform     : ANSI
 *
 * Modifications:-
 * $Log: sgxfeaturedefs.h $
 *  .
 *  --- Revision Logs Removed --- 
 *****************************************************************************/

/*
	The following features are defined:-

	SGX_FEATURE_EXTENDED_USE_ALU

	Extended ALU with support for C10, F16 and for less bank restrictions on
	internal registers.

	SGX_FEATURE_UNIFIED_TEMPS_AND_PAS

	USE temporary and primary attributes are allocated as a single block.

	SGX_FEATURE_CEM_S_USES_PROJ

	For CEM textures S needs to be iterated as the projection cordinate.

	SGX_FEATURE_HOST_PORT

	A host port is supported.

	SGX_FEATURE_DXT_TEXTURES

	DXT textures are supported.

	SGX_FEATURE_GAMMACORRECT_TEXTURES

	Gamma correction is supported on texture reads.

	SGX_FEATURE_VOLUME_TEXTURES

	Volume textures are supported.

	SGX_FEATURE_BARTLETTFILTER

	Barlett filtering is supported on PBE downscale.

	SGX_FEATURE_EDM_VERTEX_PDSADDR_FULL_RANGE

	EDM and vertex PDS program base addresses have full range.

	SGX_FEATURE_HOST_ALLOC_FROM_DPM

	The host can allocate memory from the DPM parameter buffer.

	SGX_FEATURE_MULTIPLE_MEM_CONTEXTS

	Multiple simultaneously address spaces are supported.
*/

#if defined(SGX520)
	#define SGX_CORE_FRIENDLY_NAME							"SGX520"
	#define SGX_CORE_ID										SGX_CORE_ID_520
	#define SGX_FEATURE_EXTENDED_USE_ALU
	#define SGX_FEATURE_USE_HIGH_PRECISION_FIR
	#define SGX_FEATURE_USE_LOAD_MOE_FROM_REGISTERS
	#define SGX_FEATURE_UNIFIED_TEMPS_AND_PAS
	#define SGX_FEATURE_USE_NO_INSTRUCTION_PAIRING
	#define SGX_FEATURE_CEM_S_USES_PROJ
	#define SGX_FEATURE_USE_NUMBER_OF_MUTEXES				(8)
	#define SGX_FEATURE_USE_NUMBER_OF_MONITORS				(8)
	#define SGX_FEATURE_USE_NUMBER_PC_BITS					(12)
	#define SGX_FEATURE_USE_INSTRUCTION_CACHE_LINE_BITS		(5)
	#define SGX_FEATURE_PARAMETER_BUFFER_SIZE				(0x04000000)
	#define SGX_FEATURE_ADDRESS_SPACE_SIZE					(28)
	#define SGX_FEATURE_NUM_USE_PIPES						(1)
	#define SGX_FEATURE_AUTOCLOCKGATING
	#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_8
	#define SGX_FEATURE_VERTEXDM_PIXELEMITS
	#define SGX_FEATURE_DEPTH_MANTISSA_BITS					(19)
	#define SGX_FEATURE_NUM_PDS_PIPES						(1)	
	#if defined (USE_SGX_CORE_REV_HEAD)
		#define SGX_FEATURE_ALPHATEST_SECONDARY
		#define SGX_FEATURE_ALPHATEST_COEFREORDER
		#define SGX_FEATURE_ZLS_EXTERNALZ
	#endif
	#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING)
		#define SGX_FEATURE_VDM_CONTEXT_SWITCH
		#define SGX_FEATURE_ISP_CONTEXT_SWITCH
	#endif
	#define SGX_FEATURE_UNPACKED_DPM_STATE
#else
#if defined(SGX530)
	#define SGX_CORE_FRIENDLY_NAME							"SGX530"
	#define SGX_CORE_ID										SGX_CORE_ID_530
	#define SGX_FEATURE_EXTENDED_USE_ALU
	#define SGX_FEATURE_UNIFIED_TEMPS_AND_PAS
	#define SGX_FEATURE_CEM_S_USES_PROJ
	#define SGX_FEATURE_USE_NUMBER_OF_MUTEXES				(1)
	#define SGX_FEATURE_USE_NUMBER_OF_MONITORS				(0)
	#define SGX_FEATURE_USE_NUMBER_PC_BITS					(12)
	#define SGX_FEATURE_PARAMETER_BUFFER_SIZE				(0x04000000)
	#define SGX_FEATURE_ADDRESS_SPACE_SIZE					(28)
	#define SGX_FEATURE_USE_INSTRUCTION_CACHE_LINE_BITS		(5)
	#define SGX_FEATURE_NUM_USE_PIPES						(2)
	#define SGX_FEATURE_AUTOCLOCKGATING
	#define SGX_FEATURE_VERTEXDM_PIXELEMITS	
	#define SGX_FEATURE_DEPTH_MANTISSA_BITS					(19)
	#define SGX_FEATURE_NUM_PDS_PIPES						(1)
	#if (SGX_CORE_REV == 103) || (SGX_CORE_REV == 110)
		#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_32
	#else
		#if (SGX_CORE_REV == 111) || (SGX_CORE_REV == 1111)
			#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_16_THEN_32
		#else
			#if (SGX_CORE_REV == 120) || (SGX_CORE_REV == 121) || (SGX_CORE_REV == 125) || defined (USE_SGX_CORE_REV_HEAD)
				#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_8
				#define	SGX_FEATURE_TEXTURESTRIDE_EXTENSION
			#endif
		#endif
	#endif
	#if (SGX_CORE_REV == 125)
		#define SGX_FEATURE_TEXTURE_32K_STRIDE
		#define SGX_FEATURE_PIXELBE_32K_LINESTRIDE
	#else
		#if defined (USE_SGX_CORE_REV_HEAD)
			#define SGX_FEATURE_TEXTURE_32K_STRIDE
			#define SGX_FEATURE_PIXELBE_32K_LINESTRIDE
			#define SGX_FEATURE_ALPHATEST_SECONDARY
			#define SGX_FEATURE_ALPHATEST_COEFREORDER
			#define SGX_FEATURE_ALPHATEST_SECONDARY_PERPRIMITIVE
		#endif
	#endif
	#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING)
		#define SGX_FEATURE_VDM_CONTEXT_SWITCH
		#define SGX_FEATURE_ISP_CONTEXT_SWITCH
	#endif
#else
#if defined(SGX531)
	#define SGX_CORE_FRIENDLY_NAME							"SGX531"
	#define SGX_CORE_ID										SGX_CORE_ID_531
	#define SGX_FEATURE_EXTENDED_USE_ALU
	#define SGX_FEATURE_UNIFIED_TEMPS_AND_PAS
	#define SGX_FEATURE_CEM_S_USES_PROJ
	#define SGX_FEATURE_USE_NUMBER_OF_MUTEXES				(1)
	#define SGX_FEATURE_USE_NUMBER_OF_MONITORS				(0)
	#define SGX_FEATURE_USE_NUMBER_PC_BITS					(12)
	#define SGX_FEATURE_USE_NORMALISE
	#define SGX_FEATURE_PARAMETER_BUFFER_SIZE				(0x04000000)
	#define SGX_FEATURE_ADDRESS_SPACE_SIZE					(28)
	#define SGX_FEATURE_USE_INSTRUCTION_CACHE_LINE_BITS		(5)
	#define SGX_FEATURE_NUM_USE_PIPES						(2)
	#define SGX_FEATURE_AUTOCLOCKGATING
	#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_8
	#define	SGX_FEATURE_TEXTURESTRIDE_EXTENSION
	#define SGX_FEATURE_PDS_DATA_INTERLEAVE_2DWORDS
	#define SGX_FEATURE_VERTEXDM_PIXELEMITS
	#define SGX_FEATURE_PIXELBE_32K_LINESTRIDE
	#define SGX_FEATURE_TEXTURE_32K_STRIDE
	#define SGX_FEATURE_DEPTH_MANTISSA_BITS					(19)
    #define SGX_FEATURE_ALPHATEST_SECONDARY
	#define SGX_FEATURE_ALPHATEST_COEFREORDER
	#define SGX_FEATURE_ALPHATEST_SECONDARY_PERPRIMITIVE
	#define SGX_FEATURE_USE_HIGH_PRECISION_FIR_COEFF
	#define SGX_FEATURE_ZLS_EXTERNALZ
	#define SGX_FEATURE_NUM_PDS_PIPES						(1)
	#define SGX_FEATURE_SECONDARY_REQUIRES_USE_KICK		
	#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING)
		#define SGX_FEATURE_VDM_CONTEXT_SWITCH
		#define SGX_FEATURE_ISP_CONTEXT_SWITCH
	#endif
#else
#if defined(SGX535)
	#define SGX_CORE_FRIENDLY_NAME							"SGX535"
	#define SGX_CORE_ID										SGX_CORE_ID_535
	#define SGX_FEATURE_DXT_TEXTURES
	#define SGX_FEATURE_GAMMACORRECT_TEXTURES
	#define SGX_FEATURE_VOLUME_TEXTURES
	#define SGX_FEATURE_BARTLETTFILTER
	#define SGX_FEATURE_EDM_VERTEX_PDSADDR_FULL_RANGE
	#define SGX_FEATURE_HOST_ALLOC_FROM_DPM
	#define SGX_FEATURE_MULTIPLE_MEM_CONTEXTS
	#define SGX_FEATURE_BIF_NUM_DIRLISTS					(16)
	#define SGX_FEATURE_F16_TEXTURE_FILTERING
	#define SGX_FEATURE_2D_HARDWARE
	#define SUPPORT_SGX_GENERAL_MAPPING_HEAP
	#define SGX_FEATURE_USE_NUMBER_OF_MUTEXES				(1)
	#define SGX_FEATURE_USE_NUMBER_OF_MONITORS				(0)
	#define SGX_FEATURE_USE_NUMBER_PC_BITS					(12)
	#define SGX_FEATURE_PARAMETER_BUFFER_SIZE				(0x10000000)
	#define SGX_FEATURE_ADDRESS_SPACE_SIZE					(32)
	#define SGX_FEATURE_USE_INSTRUCTION_CACHE_LINE_BITS		(4)
	#define SGX_FEATURE_NUM_USE_PIPES						(2)
	#define SGX_FEATURE_AUTOCLOCKGATING
	#define SGX_FEATURE_VERTEXDM_PIXELEMITS
	#define SGX_FEATURE_DEPTH_MANTISSA_BITS					(23)
	#define SGX_FEATURE_NUM_PDS_PIPES						(1)
	#define SGX_FEATURE_EXTENDED_USE_ALU
	#if defined (USE_SGX_CORE_REV_HEAD)
		#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_8
        #define SGX_FEATURE_ALPHATEST_SECONDARY
		#define SGX_FEATURE_ALPHATEST_COEFREORDER
		#define SGX_FEATURE_ALPHATEST_SECONDARY_PERPRIMITIVE
	#else
		#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_32	
	#endif
	#if defined (USE_SGX_CORE_REV_HEAD) || (SGX_CORE_REV == 121)
		#define SGX_FEATURE_TEXTURESTRIDE_EXTENSION
	#endif
	#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING)
		#define SGX_FEATURE_ISP_CONTEXT_SWITCH
	#endif
#else
#if defined(SGX540)
	#define SGX_CORE_FRIENDLY_NAME							"SGX540"
	#define SGX_CORE_ID										SGX_CORE_ID_540
	#define SGX_FEATURE_EXTENDED_USE_ALU
	#define SGX_FEATURE_UNIFIED_TEMPS_AND_PAS
	#define SGX_FEATURE_CEM_S_USES_PROJ
	#define SGX_FEATURE_USE_NUMBER_OF_MUTEXES				(1)
	#define SGX_FEATURE_USE_NUMBER_OF_MONITORS				(0)
	#define SGX_FEATURE_USE_NUMBER_PC_BITS					(12)
	#define SGX_FEATURE_USE_NORMALISE
	#define SGX_FEATURE_PARAMETER_BUFFER_SIZE				(0x04000000)
	#define SGX_FEATURE_ADDRESS_SPACE_SIZE					(28)
	#define SGX_FEATURE_USE_INSTRUCTION_CACHE_LINE_BITS		(5)
	#define SGX_FEATURE_NUM_USE_PIPES						(4)
	#define SGX_FEATURE_AUTOCLOCKGATING
	#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_8
	#define	SGX_FEATURE_TEXTURESTRIDE_EXTENSION
	#define SGX_FEATURE_PDS_DATA_INTERLEAVE_2DWORDS
	#define SGX_FEATURE_VERTEXDM_PIXELEMITS
	#define SGX_FEATURE_PIXELBE_32K_LINESTRIDE
	#define SGX_FEATURE_TEXTURE_32K_STRIDE
	#define SGX_FEATURE_DEPTH_MANTISSA_BITS					(19)
    #define SGX_FEATURE_ALPHATEST_SECONDARY
	#define SGX_FEATURE_ALPHATEST_COEFREORDER
	#define SGX_FEATURE_ALPHATEST_SECONDARY_PERPRIMITIVE
	#define SGX_FEATURE_USE_HIGH_PRECISION_FIR_COEFF
	#define SGX_FEATURE_ZLS_EXTERNALZ
	#define SGX_FEATURE_NUM_PDS_PIPES						(1)
	#define SGX_FEATURE_SECONDARY_REQUIRES_USE_KICK		
	#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING)
		#define SGX_FEATURE_VDM_CONTEXT_SWITCH
		#define SGX_FEATURE_ISP_CONTEXT_SWITCH
	#endif
	#if defined(USE_SGX_CORE_REV_HEAD) || (SGX_CORE_REV == 110)
	#define SGX_FEATURE_MULTI_EVENT_KICK
	#endif
#else
#if defined(SGX541)
	#define SGX_CORE_FRIENDLY_NAME							"SGX541"
	#define SGX_CORE_ID										SGX_CORE_ID_541
	#define SGX_FEATURE_EXTENDED_USE_ALU
	#define SGX_FEATURE_UNIFIED_TEMPS_AND_PAS
	#define SGX_FEATURE_CEM_S_USES_PROJ
	#define SGX_FEATURE_USE_NUMBER_OF_MUTEXES				(1)
	#define SGX_FEATURE_USE_NUMBER_OF_MONITORS				(0)
	#define SGX_FEATURE_USE_NUMBER_PC_BITS					(12)
	#define SGX_FEATURE_MULTIPLE_MEM_CONTEXTS
	#define SGX_FEATURE_BIF_NUM_DIRLISTS					(8)
	#define SGX_FEATURE_USE_NORMALISE
	#define SGX_FEATURE_PARAMETER_BUFFER_SIZE				(0x04000000)
	#define SGX_FEATURE_ADDRESS_SPACE_SIZE					(32)
	#define SGX_FEATURE_USE_INSTRUCTION_CACHE_LINE_BITS		(5)
	#define SGX_FEATURE_NUM_USE_PIPES						(4)
	#define SGX_FEATURE_AUTOCLOCKGATING
	#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_8
	#define	SGX_FEATURE_TEXTURESTRIDE_EXTENSION
	#define SGX_FEATURE_PDS_DATA_INTERLEAVE_2DWORDS
	#define SGX_FEATURE_VERTEXDM_PIXELEMITS
	#define SGX_FEATURE_PIXELBE_32K_LINESTRIDE
	#define SGX_FEATURE_TEXTURE_32K_STRIDE
	#define SGX_FEATURE_DEPTH_MANTISSA_BITS					(19)
    #define SGX_FEATURE_ALPHATEST_SECONDARY
	#define SGX_FEATURE_ALPHATEST_COEFREORDER
	#define SGX_FEATURE_ALPHATEST_SECONDARY_PERPRIMITIVE
	#define SGX_FEATURE_USE_HIGH_PRECISION_FIR_COEFF
	#define SGX_FEATURE_ZLS_EXTERNALZ
	#define SGX_FEATURE_NUM_PDS_PIPES						(1)
	#define SGX_FEATURE_SECONDARY_REQUIRES_USE_KICK
	#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING)
		#define SGX_FEATURE_VDM_CONTEXT_SWITCH_REV_2
	#endif
	#define SGX_FEATURE_MTE_STATE_FLUSH
	#define SGX_FEATURE_SPM_MODE_0
	#define SGX_FEATURE_MULTI_EVENT_KICK
#else
#if defined(SGX543)
	#define SGX_CORE_FRIENDLY_NAME							"SGX543"
	#define SGX_CORE_ID										SGX_CORE_ID_543
	#define SGX_FEATURE_EXTENDED_USE_ALU
	#define SGX_FEATURE_UNIFIED_TEMPS_AND_PAS
	#define SGX_FEATURE_CEM_S_USES_PROJ
	#define SGX_FEATURE_USE_NO_INSTRUCTION_PAIRING
	#define SGX_FEATURE_USE_UNLIMITED_PHASES
	#define SGX_FEATURE_USE_NUMBER_OF_MUTEXES				(1)
	#define SGX_FEATURE_USE_NUMBER_OF_MONITORS				(0)
	#define SGX_FEATURE_USE_NUMBER_PC_BITS					(20)
	#define SGX_FEATURE_MULTIPLE_MEM_CONTEXTS
	#define SGX_FEATURE_BIF_NUM_DIRLISTS					(8)
	#define SGX_FEATURE_TAG_UNPACK_RESULT
	#define SGX_FEATURE_PARAMETER_BUFFER_SIZE				(0x04000000)
	#define SGX_FEATURE_ADDRESS_SPACE_SIZE					(32)
	#define SGX_FEATURE_USE_INSTRUCTION_CACHE_LINE_BITS		(4)
	#define SGX_FEATURE_NUM_USE_PIPES						(4)
	#define SGX_FEATURE_AUTOCLOCKGATING
	#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_8
	#define	SGX_FEATURE_TEXTURESTRIDE_EXTENSION
	#define SGX_FEATURE_PDS_DATA_INTERLEAVE_2DWORDS
	#define SGX_FEATURE_VERTEXDM_PIXELEMITS
	#define SGX_FEATURE_PIXELBE_32K_LINESTRIDE
	#define SGX_FEATURE_TEXTURE_32K_STRIDE
	#define SGX_FEATURE_DEPTH_MANTISSA_BITS					(19)
	#define SGX_FEATURE_ALPHATEST_SECONDARY
	#define SGX_FEATURE_ALPHATEST_COEFREORDER
	#define SGX_FEATURE_ALPHATEST_SECONDARY_PERPRIMITIVE
	#define SGX_FEATURE_USE_HIGH_PRECISION_FIR_COEFF
	#define SGX_FEATURE_ZLS_EXTERNALZ
	#define SGX_FEATURE_NUM_PDS_PIPES						(1)
	#define SGX_FEATURE_SECONDARY_REQUIRES_USE_KICK
	#define SGX_FEATURE_PLANAR_YUV
	#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING)
		#define SGX_FEATURE_VDM_CONTEXT_SWITCH_REV_2
	#endif
	#define SGX_FEATURE_MTE_STATE_FLUSH
	#define SGX_FEATURE_SPM_MODE_0
	#define SGX_FEATURE_USE_SMP_REDUCEDREPEATCOUNT
	#define SGX_FEATURE_MULTI_EVENT_KICK
#else
#if defined(SGX545)
	#define SGX_CORE_FRIENDLY_NAME							"SGX545"
	#define SGX_CORE_ID										SGX_CORE_ID_545
	#define SGX_FEATURE_EXTENDED_USE_ALU
	#define SGX_FEATURE_USE_PER_INST_MOE_INCREMENTS
	#define SGX_FEATURE_USE_FCLAMP
	#define SGX_FEATURE_USE_SQRT_SIN_COS
	#define SGX_FEATURE_USE_32BIT_INT_MAD
	#define SGX_FEATURE_USE_INT_DIV
	#define SGX_FEATURE_USE_DUAL_ISSUE
	#define SGX_FEATURE_USE_EXTENDED_LOAD
	#define SGX_FEATURE_USE_NEW_EFO_OPTIONS
	#define SGX_FEATURE_USE_PACK_MULTIPLE_ROUNDING_MODES
	#define SGX_FEATURE_USE_NO_INSTRUCTION_PAIRING
	#define SGX_FEATURE_USE_UNLIMITED_PHASES
	#define SGX_FEATURE_USE_NUMBER_OF_MUTEXES				(8)
	#define SGX_FEATURE_USE_NUMBER_OF_MONITORS				(16)
	#define SGX_FEATURE_USE_NUMBER_PC_BITS					(18)
	#define SGX_FEATURE_USE_ALPHATOCOVERAGE
	#define SGX_FEATURE_USE_FMAD16_SWIZZLES
	#define SGX_FEATURE_DXT_TEXTURES
	#define SGX_FEATURE_GAMMACORRECT_TEXTURES
	#define SGX_FEATURE_VOLUME_TEXTURES
	#define SGX_FEATURE_EDM_VERTEX_PDSADDR_FULL_RANGE
	#define SGX_FEATURE_HOST_ALLOC_FROM_DPM
	#define SGX_FEATURE_F16_TEXTURE_FILTERING
	#define SGX_FEATURE_MULTIPLE_MEM_CONTEXTS
	#define SGX_FEATURE_BIF_NUM_DIRLISTS					(16)
	#define SGX_FEATURE_TAG_UNPACK_RESULT
	#define SGX_FEATURE_TAG_PCF
	#define SGX_FEATURE_TAG_RAWSAMPLE
	#define SGX_FEATURE_PARAMETER_BUFFER_SIZE				(0x10000000)
	#define SGX_FEATURE_ADDRESS_SPACE_SIZE					(32)
	#define SGX_FEATURE_USE_INSTRUCTION_CACHE_LINE_BITS		(4)
	#define SGX_FEATURE_NUM_USE_PIPES						(4)
	#define SGX_FEATURE_VERTEXDM_PIXELEMITS
	#define SGX_FEATURE_AUTOCLOCKGATING
	#define SGX_FEATURE_TEXTURE_STRIDE_GRANULARITY_8
	#define	SGX_FEATURE_TEXTURESTRIDE_EXTENSION
	#define SGX_FEATURE_PDS_DATA_INTERLEAVE_2DWORDS
	#define SGX_FEATURE_VCB
	#define SGX_FEATURE_MULTISAMPLE_2X
	#define SGX_FEATURE_HYBRID_TWIDDLING
	#define SGX_FEATURE_MONOLITHIC_UKERNEL
	#define SGX_FEATURE_DEPTH_MANTISSA_BITS					(23)
	#define SGX_FEATURE_ZLS_EXTERNALZ
	#define SGX_FEATURE_CONVOLUTION_FILTERING
	#define SGX_FEATURE_HW_LINESTRIPS
	#define SGX_FEATURE_MULTISAMPLE_TEXTURES
	#define	SGX_FEATURE_CULL_PLANES
	#define SGX_FEATURE_1D_TEXTURES
	#define SGX_FEATURE_PLANAR_YUV
	#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING)
		#if defined(USE_SGX_CORE_REV_HEAD)
			#define SGX_FEATURE_FAST_RENDER_CONTEXT_SWITCH
		#endif
		#define SGX_FEATURE_VDM_CONTEXT_SWITCH_REV_2
		#define SGX_FEATURE_ISP_CONTEXT_SWITCH_REV_2
	#endif
	#define SGX_FEATURE_ISP_BREAKPOINT_RESUME_REV_2
	#define SGX_FEATURE_NUM_PDS_PIPES						(2)
	#define SGX_FEATURE_NATIVE_BACKWARD_BLIT
	#define SGX_FEATURE_TAG_GAMMA_UNPACK_TO_F16
	// #define SGX_FEATURE_RENDER_TARGET_ARRAYS
	#define SGX_FEATURE_MAX_TA_RENDER_TARGETS				(512)
	#define SGX_FEATURE_SPM_MODE_0
	// #define SGX_FEATURE_OVERLAPPED_SPM
    // #define SGX_FEATURE_ALPHATEST_SECONDARY
	// #define SGX_FEATURE_ALPHATEST_COEFREORDER
	// #define SGX_FEATURE_ALPHATEST_SECONDARY_PERPRIMITIVE
	#define SGX_FEATURE_GAMMACORRECT_YUV_TEXTURES
	#define SGX_FEATURE_SECONDARY_REQUIRES_USE_KICK
	#define SGX_FEATURE_DCU
	// #define SGX_FEATURE_YUV_TWIDDLING
	#define SGX_FEATURE_NONPERSPECTIVE_CLIPPING
	#define SGX_FEATURE_COMPLEX_GEOMETRY_REV_2
	#if defined(USE_SGX_CORE_REV_HEAD)
		#define SGX_FEATURE_MTE_STATE_FLUSH
	#endif
	#define SGX_FEATURE_USE_LDST_EXTENDED_CACHE_MODES
	#define SGX_FEATURE_USE_SMP_REDUCEDREPEATCOUNT
	#define SGX_FEATURE_MULTI_EVENT_KICK
#endif
#endif
#endif
#endif
#endif
#endif
#endif
#endif

/* define default MP core count */
#if defined(SGX_FEATURE_MP)
#if !defined(SGX_FEATURE_MP_CORE_COUNT)
#error SGX_FEATURE_MP_CORE_COUNT must be defined when SGX_FEATURE_MP is defined
#endif
#else
#define SGX_FEATURE_MP_CORE_COUNT	(1)
#endif

#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING) && !defined(SUPPORT_SGX_PRIORITY_SCHEDULING)
	#define SUPPORT_SGX_PRIORITY_SCHEDULING
#endif

#if defined(FIX_HW_BRN_26620) && defined(SGX_FEATURE_SYSTEM_CACHE) && !defined(SGX_FEATURE_MULTI_EVENT_KICK)
	#define SGX_BYPASS_SYSTEM_CACHE
#endif

/*
	'switch-off' features if defined BRNs affect the feature
*/
#if !defined(SGX_DONT_SWITCH_OFF_FEATURES)

#if defined(FIX_HW_BRN_22199) || defined(FIX_HW_BRN_22304)
#undef SGX_FEATURE_F16_TEXTURE_FILTERING
#endif

#if defined(FIX_HW_BRN_21049) 		\
	|| defined(FIX_HW_BRN_21500) 	\
	|| defined(FIX_HW_BRN_21387) 	\
	|| defined(FIX_HW_BRN_21183) 	\
	|| defined(FIX_HW_BRN_22067)	\
	|| defined(FIX_HW_BRN_22380)	\
	|| defined(FIX_HW_BRN_22693)	\
	|| defined(FIX_HW_BRN_24435)	\
	|| defined(FIX_HW_BRN_26711)
#undef SGX_FEATURE_AUTOCLOCKGATING
#endif

#if defined(SUPPORT_SGX_LOW_LATENCY_SCHEDULING)
	#if defined(FIX_HW_BRN_26246)
		#undef SGX_FEATURE_ISP_CONTEXT_SWITCH
		#define SGX_FEATURE_SW_ISP_CONTEXT_SWITCH
	#endif
	#if defined(FIX_HW_BRN_26656)
		#undef	SGX_FEATURE_VDM_CONTEXT_SWITCH
	#endif
#endif

#if defined(FIX_HW_BRN_26656)
#undef SGX_FEATURE_MTE_STATE_FLUSH
#endif

#if defined(FIX_HW_BRN_22997) || defined(FIX_HW_BRN_22634)
#undef SGX_FEATURE_MULTITHREADED_UKERNEL
#endif

#if defined(FIX_HW_BRN_21790) || defined(FIX_HW_BRN_22181) || defined(FIX_HW_BRN_22030) || defined(FIX_HW_BRN_23054) || defined(FIX_HW_BRN_25941) || defined(FIX_HW_BRN_26336)
#undef SGX_FEATURE_VERTEXDM_PIXELEMITS
#endif

#if defined(FIX_HW_BRN_24281)
#undef SGX_FEATURE_ALPHATEST_SECONDARY
#endif

#if defined(FIX_HW_BRN_25089)
#undef SGX_FEATURE_ALPHATEST_COEFREORDER
#endif

#endif /* SGX_DONT_SWITCH_OFF_FEATURES */

#include "img_types.h"

#include "sgxcoretypes.h"

/*
	Code including this file can request a table mapping core IDs to
	the features that are supported so they can support multiple
	cores in one binary.
*/
#if defined(INCLUDE_SGX_FEATURE_TABLE)

#define SGX_FEATURE_FLAGS_USE_NO_INSTRUCTION_PAIRING	(0x00000001)
#define SGX_FEATURE_FLAGS_USE_HIGH_PRECISION_FIR		(0x00000002)
#define SGX_FEATURE_FLAGS_USE_LOAD_MOE_FROM_REGISTERS	(0x00000004)
#define SGX_FEATURE_FLAGS_USE_PER_INST_MOE_INCREMENTS	(0x00000008)
#define SGX_FEATURE_FLAGS_USE_FCLAMP					(0x00000010)
#define SGX_FEATURE_FLAGS_USE_SQRT_SIN_COS				(0x00000020)
#define SGX_FEATURE_FLAGS_USE_32BIT_INT_MAD				(0x00000040)
#define SGX_FEATURE_FLAGS_USE_INT_DIV					(0x00000080)
#define SGX_FEATURE_FLAGS_USE_DUAL_ISSUE				(0x00000100)
#define SGX_FEATURE_FLAGS_USE_EXTENDED_LOAD				(0x00000200)
#define SGX_FEATURE_FLAGS_USE_NEW_EFO_OPTIONS			(0x00000400)
#define SGX_FEATURE_FLAGS_USE_PACK_MULTIPLE_ROUNDING_MODES \
														(0x00000800)
#define SGX_FEATURE_FLAGS_USE_UNLIMITED_PHASES			(0x00001000)
#define SGX_FEATURE_FLAGS_USE_ALPHATOCOVERAGE			(0x00002000)
#define SGX_FEATURE_FLAGS_USE_UNIFIED_TEMPS_AND_PAS		(0x00004000)
#define SGX_FEATURE_FLAGS_USE_FMAD16_SWIZZLES			(0x00008000)
#define SGX_FEATURE_FLAGS_TAG_UNPACK_RESULT				(0x00010000)
#define SGX_FEATURE_FLAGS_USE_FNORMALISE				(0x00020000)
#define SGX_FEATURE_FLAGS_TAG_PCF						(0x00040000)
#define SGX_FEATURE_FLAGS_TAG_RAWSAMPLE					(0x00080000)
#define SGX_FEATURE_FLAGS_USE_NOPTRIGGER				(0x00100000)
#define SGX_FEATURE_FLAGS_USE_BRANCH_EXCEPTION			(0x00200000)
#define SGX_FEATURE_FLAGS_USE_BRANCH_EXTSYNCEND			(0x00400000)
#define SGX_FEATURE_FLAGS_USE_STORE_MOE_TO_REGISTERS	(0x00800000)
#define SGX_FEATURE_FLAGS_USE_CFI						(0x01000000)
#define SGX_FEATURE_FLAGS_VCB							(0x02000000)
#define SGX_FEATURE_FLAGS_EXTERNAL_LDST_SMP_UNIT		(0x04000000)
#define SGX_FEATURE_FLAGS_USE_VEC34						(0x08000000)
#define SGX_FEATURE_FLAGS_TAG_GAMMA_UNPACK_TO_F16		(0x10000000)
#define SGX_FEATURE_FLAGS_USE_BRANCH_PWAIT				(0x20000000)
#define SGX_FEATURE_FLAGS_TAG_GRAD_SAMPLE_IN_DFC		(0x40000000)
#define SGX_FEATURE_FLAGS_TAG_RAWSAMPLEBOTH				(0x80000000)

#define SGX_FEATURE_FLAGS2_USE_SMP_REDUCEDREPEATCOUNT	(0x00000001)
#define SGX_FEATURE_FLAGS2_USE_LDST_QWORD				(0x00000002)
#define SGX_FEATURE_FLAGS2_USE_IDXSC					(0x00000004)
#define SGX_FEATURE_FLAGS2_USE_TEST_SABLND				(0x00000008)
#define SGX_FEATURE_FLAGS2_USE_LDRSTR_REPEAT			(0x00000010)
#define SGX_FEATURE_FLAGS2_USE_STR_PREDICATE			(0x00000020)
#define SGX_FEATURE_FLAGS2_USE_LDRSTR_EXTENDED_IMMEDIATE	\
														(0x00000040)
#define SGX_FEATURE_FLAGS2_USE_TWO_INDEX_BANKS			(0x00000080)
#define SGX_FEATURE_FLAGS2_USE_SPRVV					(0x00000100)
#define SGX_FEATURE_FLAGS2_USE_LDST_EXTENDED_CACHE_MODES	\
														(0x00000200)
#define SGX_FEATURE_FLAGS2_USE_ST_NO_READ_BEFORE_WRITE	\
														(0x00000400)
#define SGX_FEATURE_FLAGS2_USE_SMP_RESULT_FORMAT_CONVERT	\
														(0x00000800)

#endif /* defined(INCLUDE_SGX_FEATURE_TABLE) */

/******************************************************************************
 End of file (sgxfeaturedefs.h)
******************************************************************************/
